# /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: Tomas Jirka
@summary: QAStats API functions
"""
import json
import re
import requests
import os

from shishito.reporting.reporter import Reporter
from shishito.runtime.shishito_support import ShishitoSupport


class QAStats(object):
    """ QAStats object """

    def __init__(self, user, password, timestamp, epoch, build):
        self.shishito_support = ShishitoSupport()
        self.qastats_base_url = self.shishito_support.get_opt('qastats_url')
        self.user = user
        self.password = password
        self.timestamp = timestamp
        self.epoch = epoch
        self.build = build

        # project specific config
        self.project_id = self.shishito_support.get_opt('qastats_project_id')

        # shishito results
        self.reporter = Reporter()
        self.shishito_results = self.reporter.get_xunit_test_cases(timestamp)

        self.default_headers = {'Content-Type': 'application/json'}
        self.result_url = self.qastats_base_url + '/api/v1/results'
        self.project_url = self.shishito_support.get_opt('base_url')

    def post_results(self):
        """ Create test-cases on QAStats, adds a new test run and update results for the run
            {
               "project_id": 123,
               "timestamp": 1470133472,
               "build": "773",              // optional
               "environment": "Firefox"     // optional
               "branch": "develop",         // optional
               "git": "ae232a",             // optional
               "results": [
                  { "test": "test_login", "result": "pass" },   // [pass fail err nr]
                  ...
               ],
            }
        """

        for (i, run) in enumerate(self.shishito_results):
            environment = run['name'];
            m =re.match('^(.*)\.xml$', environment)
            if m != None: environment = m.group(1)

            payload = {
                    'project_id': self.project_id,
                    'timestamp': self.epoch + i,
                    'environment': environment
            }
            if self.build:
                payload['build'] = self.build
            if 'QA_BRANCH_TO_TEST' in os.environ:
                payload['branch'] = os.environ['QA_BRANCH_TO_TEST']
            if 'QA_GIT_COMMIT' in os.environ:
                payload['git'] = os.environ["QA_GIT_COMMIT"]
            elif 'CIRCLE_REPOSITORY_URL' in os.environ:
                print('github url is known')
                payload['git'] = os.environ['CIRCLE_REPOSITORY_URL']
            if 'CIRCLE_TEST_REPORTS' in os.environ:
                print('result url is known')
                payload['reporturl'] = os.environ.get('CIRCLE_TEST_REPORTS')
            if self.project_url is not None:
                payload['testurl']=self.project_url
            status_map = {
                'error': 'err',
                'failure': 'fail',
                'success': 'pass',
                'skipped': 'nr'
            }
            results = [ {'test': t['name'], 'result': status_map[t['result']]} for t in run['cases'] ]
            payload['results'] = results
            print(payload)
            json_payload = json.dumps(payload)
            r = requests.post(self.result_url, auth=(self.user, self.password), data=json_payload,
                                 headers=self.default_headers)

            if r.status_code == requests.codes.ok:
                try:
                    resp = r.json()
                    if 'result' in resp and resp['result'] == 'OK':
                        print("Results uploaded to QAStats")
                        return True
                except (ValueError, AttributeError):
                    pass

            print("Error: uploading tests to QAStats\n\n", json_payload, "\n")
            print("\tStatus-code:\t" + str(r.status_code) + "\n")
            for n, v in r.headers.items():
                print("\t" + n + "\t" + v)
            print("")
            print(r.text)


from shishito.runtime.platform.shishito_control_test import ShishitoControlTest


class ControlTest(ShishitoControlTest):
    """ ControlTest for generic platform """

    def start_browser(self):
        """ Webdriver startup function.

        :return: initialized webdriver
        """

    def start_test(self, reload_page=None):
        """ To be executed before every test-case (test function).

        :param reload_page:
        """

    def stop_browser(self):
        """ Webdriver termination function. """

    def stop_test(self, test_info):
        """ To be executed after every test-case (test function). If test failed, function saves
        screenshots created during test.

        :param test_info: information about test
        """



from shishito.runtime.platform.shishito_execution import ShishitoExecution


class ControlExecution(ShishitoExecution):
    """ ControlExecution for generic platform"""

    def get_test_result_prefix(self, config_section):
        """ Create string prefix for test results.

        :param str config_section: section in platform/environment.properties config
        :return: str with test result prefix
        """

        return 'generic'

    def run_tests(self):
        """ Trigger PyTest runner. Run PyTest for for generic tests. """

        return self.trigger_pytest('generic')



# /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: Vojtech Burian
"""
import requests
import json
from shishito.runtime.shishito_support import ShishitoSupport
import os


class CircleAPI(object):
    """Handles communication with Circle CI via REST API"""

    def __init__(self):
        self.shishito_support = ShishitoSupport()
        self.api_token = self.shishito_support.get_opt('circleci_api_token')
        self.circle_username = self.shishito_support.get_opt('circleci_username')
        self.circle_project = self.shishito_support.get_opt('circleci_project')
        self.circle_branch = self.shishito_support.get_opt('circleci_branch')

    def collect_artifacts(self, destination_folder):
        """downloads build artifacts from CircleCI for latest build from specific branch"""
        if not os.path.exists(destination_folder):
            os.makedirs(destination_folder)
        artifact_data = self.get_artifact_data()
        for artifact in artifact_data:
            self.save_artifact(artifact, destination_folder)
        return bool(os.listdir(destination_folder))

    def save_artifact(self, artifact, destination_folder):
        """ saves artifact into specified folder """
        file_name = artifact['url'].split('/')[-1]
        file_request_url = artifact['url'] + '?circle-token=' + self.api_token

        response = requests.get(file_request_url, stream=True)
        response.raise_for_status()

        with open(os.path.join(destination_folder, file_name), 'wb') as extension:
            for block in response.iter_content(1024):
                extension.write(block)

    def get_artifact_data(self):
        """returns json with artifact urls"""
        latest_dev_build_url = (
            'https://circleci.com/api/v1/project/{circle_username}/{circle_project}/tree/'
            '{circle_branch}?circle-token={api_token}&limit=1').format(**self.__dict__)

        headers = {'Accept': 'application/json'}
        response = requests.get(latest_dev_build_url, headers=headers)
        builds_json = json.loads(response.text)
        build_number = builds_json[0]['build_num']

        artifacts_url = (
            'https://circleci.com/api/v1/project/{circle_username}/'
            '{circle_project}/{build_number}/artifacts?circle-token={api_token}'
        ).format(build_number=build_number, **self.__dict__)

        response = requests.get(artifacts_url, headers=headers)
        return json.loads(response.text)



import json
import os
import re

from shishito.runtime.shishito_support import ShishitoSupport
from shishito.ui.selenium_support import SeleniumTest


class ShishitoControlTest(object):
    """ Base class for ControlTest objects. """

    def __init__(self):
        self.shishito_support = ShishitoSupport()

        # create control environment object
        control_env_obj = self.shishito_support.get_module('test_environment')
        self.test_environment = control_env_obj(self.shishito_support)

        self.drivers = []

    def start_browser(self, base_url = None):
        """ Webdriver startup function.

        :return: initialized webdriver
        """

        config_section = self.shishito_support.get_opt('environment_configuration')

        # call browser from proper environment
        driver = self.test_environment.call_browser(config_section)
        self.drivers.append(driver)

        # load init url
        if not base_url:
            base_url = self.shishito_support.get_opt('base_url')

        if base_url:
            self.test_init(driver, base_url)
        else:
            self.test_init(driver)
        return driver

    def start_test(self, reload_page=None):
        """ To be executed before every test-case (test function).

        :param reload_page:
        """

    def stop_browser(self):
        """ Webdriver termination function. """

        for driver in self.drivers:
            driver.quit()

    def stop_test(self, test_info, debug_events=None):
        """ To be executed after every test-case (test function). If test failed, function saves
        screenshots created during test.

        :param test_info: information about test
        """

        if test_info.test_status not in ('passed', None):
            # save screenshot in case test fails
            test_name = re.sub('[^A-Za-z0-9_.]+', '_', test_info.test_name)

            # Capture screenshot and debug info from driver(s)
            for driver in self.drivers:
                if(self.shishito_support.test_platform == 'mobile'):
                    browser_name = 'appium'
                else:
                    browser_name = driver.name
                file_name = browser_name + '_' + test_name
                ts = SeleniumTest(driver)
                ts.save_screenshot(name=file_name)

                #Save debug info to file
                if debug_events is not None:
                    debugevent_folder = os.path.join(self.shishito_support.project_root, 'debug_events')

                    if not os.path.exists(debugevent_folder):
                        os.makedirs(debugevent_folder)

                    with open(os.path.join(debugevent_folder, file_name + '.json'), 'w') as logfile:
                            json.dump(debug_events, logfile)

    def test_init(self, driver, url=None):
        """ Executed only once after browser starts.
         Suitable for general pre-test logic that do not need to run before every individual test-case.

        :param WebDriver driver: driver instance to set up
        :param str url: initial URL

        """



# /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: Vojtech Burian
@summary: TestRail API functions
"""
import json

import requests

from shishito.reporting.reporter import Reporter
from shishito.runtime.shishito_support import ShishitoSupport


class TestRail(object):
    """ TestRail object """

    def __init__(self, user, password, timestamp, build):
        self.shishito_support = ShishitoSupport()
        self.test_rail_instance = self.shishito_support.get_opt('test_rail_url')
        self.user = user
        self.password = password
        self.timestamp = timestamp

        # project specific config
        self.project_id = self.shishito_support.get_opt('test_rail_project_id')
        self.section_id = self.shishito_support.get_opt('test_rail_section_id')
        self.test_plan_id = self.shishito_support.get_opt('test_rail_test_plan_id')
        self.test_plan_name = self.shishito_support.get_opt('test_rail_test_plan_name') or build
        self.suite_id = self.shishito_support.get_opt('test_rail_suite_id')

        # shishito results
        self.reporter = Reporter()
        self.shishito_results = self.reporter.get_xunit_test_cases(timestamp)

        self.default_headers = {'Content-Type': 'application/json'}
        self.uri_base = self.test_rail_instance + '/index.php?/api/v2/'

    def post_results(self):
        """ Create test-cases on TestRail, adds a new test run and update results for the run """
        self.create_missing_test_cases()

        if self.test_plan_name:
            test_plan_id = self.add_test_plan()
        else:
            test_plan_id = self.test_plan_id

        test_run = self.add_test_run(test_plan_id)
        self.add_test_results(test_run)

    def tr_get(self, url):
        """ GET request for TestRail API

        :param url: url endpoint snippet
        :return: response JSON
        """
        response = requests.get(self.uri_base + url, auth=(self.user, self.password), headers=self.default_headers)
        #print(self.uri_base + url, response, response.text)
        return response.json()

    def tr_post(self, url, payload):
        """ GET request for TestRail API

        :param url: url endpoint snippet
        :param payload: payload for the POST api call
        :return: response object
        """
        return requests.post(self.uri_base + url, auth=(self.user, self.password), data=json.dumps(payload),
                             headers=self.default_headers)

    def get_all_test_plans(self):
        """ Gets list of all test-plans from certain project

        :return: list of test-plans (names = strings)
        """
        test_plans_list = self.tr_get('get_plans/{}'.format(self.project_id))
        return [{'name': test_plan['name'], 'id': test_plan['id']} for test_plan in test_plans_list]

    def get_all_test_cases(self):
        """ Gets list of all test-cases from certain project

        :return: list of test-cases (names = strings)
        """
        test_case_list = self.tr_get('get_cases/{}&section_id={}&suite_id={}'.format(self.project_id, self.section_id, self.suite_id))
        return [{'title': test_case['title'], 'id': test_case['id']} for test_case in test_case_list]

    def create_test_case(self, title):
        """ Creates a new test case in TestRail

        :param title: Title of the test-case
        :return: response object
        """
        return self.tr_post('add_case/{}'.format(self.section_id), {"title": title})

    def create_missing_test_cases(self):
        """ Creates new test-cases on TestRail for those in test project (those run by pytest).
         Does not create test-cases if already existed on TestRail.

        :return: list of test-cases that could not be created on TestRail (post failure)
        """
        post_errors = []
        test_case_names = [item['title'] for item in self.get_all_test_cases()]
        # Iterate over results for each environment combination
        for result_combination in self.shishito_results:
            # Create TestRail entry for every test-case in combination (if missing)
            for item in result_combination['cases']:
                if item['name'] not in test_case_names:
                    response = self.create_test_case(item['name'])
                    if response.status_code != requests.codes.ok:
                        post_errors.append(item['name'])
                    else:
                        test_case_names.append(item['name'])
        return post_errors

    def add_test_plan(self):
        test_plan_id = 0

        # Check if already exists
        for plan in self.get_all_test_plans():
            if plan['name'] == self.test_plan_name:
                return plan['id']

        result = self.tr_post('add_plan/{}'.format(self.project_id), {"name": self.test_plan_name})

        return json.loads(result.text)['id']

    def add_test_run(self, test_plan_id = None):
        """ Adds new test run under certain test plan into TestRail

        :return: dictionary of TestRail run names & IDs
        """
        test_plan_id = test_plan_id or self.test_plan_id
        runs_created = []
        # Iterate over results for each environment combination
        for result_combination in self.shishito_results:
            run_name = '{} ({})'.format(result_combination['name'][:-4], self.timestamp)
            test_run = {"case_ids": [case['id'] for case in self.get_all_test_cases()]}
            result = self.tr_post('add_plan_entry/{}'.format(test_plan_id),
                                  {"suite_id": self.suite_id, "name": run_name, "runs": [test_run]}).json()
            # lookup test run id
            for run in result['runs']:
                if run['name'] == run_name:
                    runs_created.append({'combination': result_combination['name'], 'id': run['id']})
        return runs_created

    def add_test_results(self, test_runs):
        """ Add test results for specific test run based on parsed xUnit results

        :return: list of test run IDs for which results could not be added (post failure)
        """
        post_errors = []
        run_ids = {r['combination']: r['id'] for r in test_runs}
        # Iterate over results for each environment combination
        for result in self.shishito_results:
            run_id = run_ids.get(result['name'])
            if not run_id:
                continue

            test_results = []
            tr_tests = {t['title']: t['id'] for t in self.tr_get('get_tests/{}'.format(run_id))}
            # Create TestRail entry for every test-case in combination (if missing)
            for xunit_test in result['cases']:
                tr_test_id = tr_tests.get(xunit_test['name'])
                result_id = {'success': 1, 'failure': 5}.get(xunit_test['result'])
                if tr_test_id and result_id:
                    # Add result content into the payload list
                    result = {'test_id': tr_test_id, 'status_id': result_id}
                    if result_id == 5:
                        result['comment'] = xunit_test['failure_message']
                    test_results.append(result)

            response = self.tr_post('add_results/{}'.format(run_id), {'results': test_results})
            if response.status_code != requests.codes.ok:
                post_errors.append(run_id)
        return post_errors



from __future__ import absolute_import

from selenium import webdriver

from shishito.runtime.environment.shishito import ShishitoEnvironment


class ControlEnvironment(ShishitoEnvironment):
    """ Remote control environment. """

    def call_browser(self, config_section):
        """ Starts browser """

        # get browser capabilities
        capabilities = self.get_capabilities(config_section)

        # get remote url
        remote_url = self.shishito_support.get_opt('remote_driver_url')

        # get driver
        browser_type = self.shishito_support.get_opt(config_section, 'browser').lower()
        return self.start_driver(browser_type, capabilities, remote_url, config_section)



    def start_driver(self, browser_type, capabilities, remote_driver_url, config_section):
        """ Call remote browser (driver) """
        try:
            extensions = capabilities['chromeOptions']['extensions']    # do not print the huge base64 extension content
            capabilities['chromeOptions']['extensions'] = ['...']
            print("Starting remote driver", capabilities)
            capabilities['chromeOptions']['extensions'] = extensions
        except:
            print("Starting remote driver", capabilities)

        browser_profile = self.get_browser_profile(browser_type, capabilities, config_section)
        driver = webdriver.Remote(
            command_executor=remote_driver_url,
            desired_capabilities=capabilities,
            browser_profile=browser_profile)

        if 'resolution' in capabilities:
            (width, height) = capabilities['resolution'].split('x')
            driver.set_window_size(width, height)

        return driver



# /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: Vojtech Burian
@summary: Common library for functions executing actions over Ripple Emulator UI (Mobile OS web emulator)
"""
import time

from selenium.webdriver.common.by import By


class TestRipple(object):

    def __init__(self, driver):
        self.driver = driver
        self.section_locator = (By.CSS_SELECTOR, '.%s.sortable.main.ui-sortable')
        self.arrow_locator = (
            By.CSS_SELECTOR,
            '#ui > section.%s-panel-collapse.ui-state-default.ui-corner-all.ui-state-hover > span')
        self.gps_header_locator = (By.CSS_SELECTOR, '#gps-container > section.h2.info-header > section.collapse-handle')
        self.gps_container_locator = (By.CSS_SELECTOR, '#gps-container > section.info.ui-widget-content.ui-corner-all')
        self.geo_latitude_locator = (By.ID, 'geo-latitude')
        self.geo_longitude_locator = (By.ID, 'geo-longitude')

    def prepare_for_testing(self):
        """ prepares Ripple Simulator for App testing """
        self.expand_section('left', False)
        self.expand_section('right', False)
        self.switch_to_ripple_app()

    def expand_section(self, side, expand=True):
        section = self.driver.find_element(self.section_locator[0], self.section_locator[1] % side)
        arrow = self.driver.find_element(self.arrow_locator[0], self.arrow_locator[1] % side)
        contains = section.get_attribute('style') in ('left: 0px; opacity: 1;', '')
        if expand != contains:
            arrow.click()
            time.sleep(2)

    def switch_to_ripple_app(self):
        """ switches to PhoneGap HMTL app iframe (so selenium can target elements inside it) """
        app_frame = self.driver.find_elements_by_tag_name('iframe')[0]
        self.driver.switch_to_frame(app_frame)

    def switch_from_ripple_app(self):
        """ switches into the default Ripple Emulator UI DOM (so selenium can target Ripple control elements) """
        self.driver.switch_to_default_content()

    def set_geo_location(self, lat, lng):
        """ sets location to given coordinates """
        self.switch_from_ripple_app()
        self.expand_section('right', True)
        if not self.driver.find_element(*self.gps_container_locator).is_displayed():
            self.driver.find_element(*self.gps_header_locator).click()
            time.sleep(1)
        latitude = self.driver.find_element(*self.geo_latitude_locator)
        longitude = self.driver.find_element(*self.geo_longitude_locator)
        latitude.clear()
        longitude.clear()
        latitude.send_keys(str(lat))
        longitude.send_keys(str(lng))
        self.expand_section('right', False)
        self.driver.refresh()
        self.switch_to_ripple_app()



"""
@summary: Common configuration functions supporting test execution.
 Various startup and termination procedures, helper functions etc.
 Not to be used for directly testing the system under test (must not contain Asserts etc.)
"""
import os
import re
from shishito.runtime.platform.shishito_control_test import ShishitoControlTest
#import pyscreenshot as Screenshotter


class ControlTest(ShishitoControlTest):
    """ ControlTest for node-webkit platform """

    def start_browser(self):
        # call browser from proper environment
        config_section = self.shishito_support.get_opt('environment_configuration')
        print("*********** config_section =", config_section)
        self.driver = self.test_environment.call_browser(config_section)
        return self.driver

    def test_init(self):
        """ Executed only once after browser starts.
        Suitable for general pre-test logic that do not need to run before every individual test-case.
        Waiting for given time (setting "default_implicit_wait").
        """
        self.driver.implicitly_wait(int(self.shishito_support.get_opt('default_implicit_wait')))


    def stop_test(self, test_info):
        """
        !!!TEMPORARY METHOD!!! \n
        To be executed after every test-case (test function). If test failed, function saves
        screenshots created during test.

        For more information see: https://code.google.com/p/chromedriver/issues/detail?id=816

        :param test_info: information about test
        :return:
        """
        if test_info.test_status not in ('passed', None):
            # save screenshot in case test fails
            screenshot_folder = os.path.join(self.shishito_support.project_root, 'screenshots')

            if not os.path.exists(screenshot_folder):
                os.makedirs(screenshot_folder)

            file_name = re.sub('[^A-Za-z0-9_. ]+', '', test_info.test_name)
            self.driver.save_screenshot(os.path.join(screenshot_folder, file_name + '.png'))
            #Screenshotter.grab_to_file(os.path.join(screenshot_folder, file_name + '.png'))



from __future__ import absolute_import

from selenium import webdriver
import sys
import time

from shishito.runtime.environment.shishito import ShishitoEnvironment
from shishito.services.browserstack import BrowserStackAPI


class ControlEnvironment(ShishitoEnvironment):
    """ Browserstack control environment. """

    def __init__(self, shishito_support):
        super(ControlEnvironment, self).__init__(shishito_support)

        self.bs_api = BrowserStackAPI()

    def call_browser(self, config_section):
        """ Start webdriver for given config section. Wait for free browserstack session.
        Prepare capabilities for the browser. Created webdriver will be connected to browserstack.

        :param str config_section: section in platform/environment.properties config
        :return: created webdriver
        """

        # get browser stack credentials
        try:
            bs_user, bs_password = self.shishito_support.get_opt('browserstack').split(':', 1)
        except (AttributeError, ValueError):
            raise ValueError('Browserstack credentials were not specified! Unable to start browser.')

        # wait until free browserstack session is available
        if not self.bs_api.wait_for_free_sessions(
            (bs_user, bs_password),
            int(self.shishito_support.get_opt('session_waiting_time')),
            int(self.shishito_support.get_opt('session_waiting_delay'))
        ):
            sys.exit('No free browserstack session - exit.')

        # get browser capabilities
        capabilities = self.get_capabilities(config_section)

        # prepare remote driver url
        hub_url = 'http://{0}:{1}@hub.browserstack.com:80/wd/hub'.format(bs_user, bs_password)

        # get browser type
        browser_type = self.shishito_support.get_opt(config_section, 'browser')
        browser_type = browser_type.lower()

        # call remote driver
        driver = self.start_driver(browser_type, capabilities, remote_driver_url=hub_url)

        session = self.bs_api.get_session((bs_user, bs_password), capabilities['build'], 'running')
        self.session_link = self.bs_api.get_session_link(session)
        self.session_id = self.bs_api.get_session_hashed_id(session)

        return driver

    def get_pytest_arguments(self, config_section):
        """ Get environment specific arguments for pytest.

        :param config_section: section in platform/environment.properties config
        :return: dict with arguments for pytest or None
        """
        test_platform = self.shishito_support.test_platform
        arguments = {}
        if(test_platform == 'web'):
            browser = self.shishito_support.get_opt(config_section, 'browser')
            browser_version = self.shishito_support.get_opt(config_section, 'browser_version')
            os_type = self.shishito_support.get_opt(config_section, 'os')
            os_version = self.shishito_support.get_opt(config_section, 'os_version')
            resolution = self.shishito_support.get_opt(config_section, 'resolution')
            test_result_prefix = '[%s, %s, %s, %s, %s]' % (
            browser, browser_version, os_type, os_version, resolution
            )

            # Add browserstack credentials
            bs_auth = self.shishito_support.get_opt('browserstack')

            # prepare pytest arguments into execution list
            return {
                '--junit-prefix=': '--junit-prefix=' + test_result_prefix,
                '--html-prefix=': '--html-prefix=' + test_result_prefix,
                '--browser=': '--browser=' + browser,
                '--browser_version=': '--browser_version=' + browser_version,
                '--os=': '--os=' + os_type,
                '--os_version=': '--os_version=' + os_version,
                '--resolution=': '--resolution=' + resolution,
                '--browserstack=': '--browserstack=' + bs_auth
                }
        if(test_platform == 'mobile'):
            browser = self.shishito_support.get_opt(config_section, 'browser')
            platform = self.shishito_support.get_opt(config_section, 'platform')
            device = self.shishito_support.get_opt(config_section, 'device')
            deviceOrientation = self.shishito_support.get_opt(config_section, 'deviceOrientation') or 'portrait'

            test_result_prefix = '[%s, %s, %s]' % (
                browser, platform, device
            )

            # Add browserstack credentials
            bs_auth = self.shishito_support.get_opt('browserstack')

            # prepare pytest arguments into execution list
            return {
                '--junit-prefix=': '--junit-prefix=' + test_result_prefix,
                '--html-prefix=': '--html-prefix=' + test_result_prefix,
                '--browser=': '--browser=' + browser,
                '--platform=': '--platform=' + platform,
                '--device=': '--device=' + device,
                '--deviceOrientation=': '--deviceOrientation=' + deviceOrientation,
                '--browserstack=': '--browserstack=' + bs_auth
            }

    def get_capabilities(self, config_section):
        """ Return dictionary of capabilities for specific config combination.

        :param str config_section: section in platform/environment.properties config
        :return: dict with capabilities
        """

        test_platform = self.shishito_support.test_platform
        get_opt = self.shishito_support.get_opt

        default_capabilities = super().get_capabilities(config_section)
        special_capabilities = {}
        capabilities = {
            'browserstack.debug': get_opt('browserstack_debug').lower(),
            'project': get_opt('project_name'),
            'build': get_opt('build_name'),
            'name': self.get_test_name() + time.strftime('_%Y-%m-%d'),
            'browserstack.local': get_opt('browserstack_local') or False
             }
        if(test_platform == 'web'):
            special_capabilities = {
                'os': get_opt(config_section, 'os'),
                'os_version': get_opt(config_section, 'os_version'),
                'browser': get_opt(config_section, 'browser'),
                'browser_version': get_opt(config_section, 'browser_version'),
                'resolution': get_opt(config_section, 'resolution')
            }
        if(test_platform == 'mobile'):
            special_capabilities = {
                'browser': get_opt(config_section, 'browser'),
                'platform': get_opt(config_section, 'platform'),
                'device': get_opt(config_section, 'device'),
                'deviceOrientation': get_opt(config_section, 'deviceOrientation') or 'portrait'
            }
        self.add_cmdline_arguments_to_browser(capabilities, config_section)
        return {**default_capabilities, **capabilities, **special_capabilities}
    # TODO will need to implement some edge cases from there (mobile emulation etc..)
    # def get_browser_profile(self, browser_type):
    # """ returns ChromeOptions or FirefoxProfile with default settings, based on browser """
    # if browser_type.lower() == 'chrome':
    #         profile = webdriver.ChromeOptions()
    #         profile.add_experimental_option("excludeSwitches", ["ignore-certificate-errors"])
    #         # set mobile device emulation
    #         mobile_browser_emulation = self.gid('mobile_browser_emulation')
    #         if mobile_browser_emulation:
    #             profile.add_experimental_option("mobileEmulation", {"deviceName": mobile_browser_emulation})
    #         return profile
    #     elif browser_type.lower() == 'firefox':
    #         return webdriver.FirefoxProfile()
    #
    # def update_browser_profile(self, capabilities, browser_type=None):
    #     """ Returns updated browser profile ready to be passed to driver """
    #     browser_profile = None
    #     test_mobile = self.gid('test_mobile')
    #     appium_platform = self.gid('appium_platform')
    #     if test_mobile != 'yes' and appium_platform not in ('android', 'ios'):
    #         if browser_type is None:
    #             browser_type = capabilities['browser'].lower()
    #         browser_profile = self.get_browser_profile(browser_type)
    #
    #         # add extensions to remote driver
    #         if self.gid('with_extension'):
    #             browser_profile = self.add_extension_to_browser(browser_type, browser_profile)
    #
    #         # add Chrome options to desired capabilities
    #         if browser_type == 'chrome':
    #             chrome_capabilities = browser_profile.to_capabilities()
    #             capabilities.update(chrome_capabilities)
    #             browser_profile = None
    #     return browser_profile

    def start_driver(self, browser_type, capabilities, remote_driver_url):
        """ Prepare selenium webdriver.

        :param browser_type: type of browser for which prepare driver
        :param capabilities: capabilities used for webdriver initialization
        :param remote_driver_url: browserstack url, to which the driver will be connected
        """

        browser_profile = self.get_browser_profile(browser_type, capabilities)

        driver = webdriver.Remote(
            command_executor=remote_driver_url,
            desired_capabilities=capabilities,
            browser_profile=browser_profile)

        return driver



from __future__ import absolute_import

from selenium import webdriver

from shishito.runtime.environment.shishito import ShishitoEnvironment


class ControlEnvironment(ShishitoEnvironment):
    """ Local control environment. """

    def get_capabilities(self, config_section):
        """ Return dictionary of capabilities for specific config combination.
        :param str config_section: section in platform/environment.properties config
        :return: dict with capabilities
        """

        get_opt = self.shishito_support.get_opt
        default_capabilities = super().get_capabilities(config_section)
        capabilities = {
            'marionette': str(get_opt('firefox_marionette')).lower() == 'true',
        }
        return {**default_capabilities, **capabilities}


    def start_driver(self, browser_type, capabilities, config_section):
        """ Prepare selenium webdriver.

        :param browser_type: type of browser for which prepare driver
        :param capabilities: capabilities used for webdriver initialization
        """

        # get browser profile
        browser_profile = self.get_browser_profile(browser_type, capabilities, config_section)

        # starts local browser
        if browser_type == "firefox":
            from selenium.webdriver.firefox.options import Options
            firefox_options = Options()
            for arg in self.get_browser_arguments(config_section):
                firefox_options.add_argument(arg)
            driver = webdriver.Firefox(browser_profile, desired_capabilities=capabilities, firefox_options=firefox_options)
        elif browser_type == "chrome":
            driver = webdriver.Chrome(desired_capabilities=capabilities, chrome_options=browser_profile)
        elif browser_type == "ie":
            driver = webdriver.Ie(capabilities=capabilities)
        elif browser_type == "phantomjs":
            driver = webdriver.PhantomJS(desired_capabilities=capabilities)
        elif browser_type == "opera":
            driver = webdriver.Opera(desired_capabilities=capabilities)
            # SafariDriver bindings for Python not yet implemented
            # elif browser == "Safari":
            # self.driver = webdriver.SafariDriver()
        else:
            raise ValueError('Unknown type of browser.')

        return driver


    def call_browser(self, config_section):
        """ Start webdriver for given config section. Prepare capabilities for the browser, set browser resolution.

        :param str config_section: section in platform/environment.properties config
        :return: created webdriver
        """

        # get browser capabilities
        capabilities = self.get_capabilities(config_section)

        # get browser type
        browser_type = self.shishito_support.get_opt(config_section, 'browser').lower()

        # get driver
        driver = self.start_driver(browser_type, capabilities, config_section=config_section)

        # set browser size is defined
        browser_size = self.shishito_support.get_opt(config_section, 'resolution')
        if browser_size:
            # default size --> leave it on webdriver
            width, height = browser_size.split('x')
            driver.set_window_size(width, height)

        return driver




# /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: Vojtech Burian
@summary: Common selenium webdriver related functions.
Helper functions that abstract often basic webdriver operations into more usable functional blocks.
"""
import inspect
import time
import os
import glob

import requests
from selenium.webdriver.support.ui import Select
from selenium.common.exceptions import NoSuchElementException, \
    ElementNotVisibleException, TimeoutException
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys
from unittestzero import Assert
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.action_chains import ActionChains

from shishito.runtime.shishito_support import ShishitoSupport


class SeleniumTest(object):
    def __init__(self, driver):
        self.driver = driver
        self.shishito_support = ShishitoSupport()
        self.base_url = self.shishito_support.get_opt('base_url')
        self.default_implicit_wait = int(self.shishito_support.get_opt('default_implicit_wait'))
        self.timeout = int(self.shishito_support.get_opt('timeout'))

    def save_screenshot(self, name=None, project_root=None):
        """ Saves application screenshot """
        if not name:
            # Use the name of browser and caller function (e.g. 'chrome_test_google_search'
            name = self.driver.name + "_" + inspect.stack()[1][3]
        if not project_root:
            project_root = self.shishito_support.project_root
        screenshot_folder = os.path.join(project_root, 'screenshots')
        if not os.path.exists(screenshot_folder):
            os.makedirs(screenshot_folder)

        existing_images = glob.glob(os.path.join(screenshot_folder, name + '_*.png'))
        actual_pic_nr = len(existing_images) + 1
        self.driver.save_screenshot(os.path.join(screenshot_folder, '{}_{}.png'.format(name, actual_pic_nr)))

    def save_file_from_url(self, file_path, url):
        """ Saves file from url """
        if os.path.isfile(file_path):
            print('File %s already exists.' % file_path)
            return

        response = requests.get(url, stream=True)
        response.raise_for_status()

        with open(file_path, 'wb') as save_file:
            for block in response.iter_content(1024):
                if not block:
                    break
                save_file.write(block)

    # Deprecated use property directly
    def get_base_url(self):
        return self.base_url

    # Deprecated use property directly
    def get_current_url(self):
        return self.current_url

    @property
    def current_url(self):
        """ Return the url for the current page."""
        return self.driver.current_url

    def hover_on(self, element):
        """ Mouse over specific element """
        mouse_over = ActionChains(self.driver).move_to_element(element)
        mouse_over.perform()

    def go_to_page(self, url):
        """ Opens url in currently active window """
        self.driver.get(url)
        self.driver.implicitly_wait(self.default_implicit_wait)

    def click_and_wait(self, element, locator=None):
        """ clicks on a element and then waits for specific element to be present or simply waits implicitly """
        element.click()
        if locator:
            self.wait_for_element_ready(locator)
        else:
            self.driver.implicitly_wait(10)

    def check_images_are_loaded(self):
        """ checks all images on the pages and verifies if they are properly loaded """
        script = 'return arguments[0].complete && typeof arguments[0].naturalWidth' \
                 ' != "undefined" && arguments[0].naturalWidth > 0'
        images_not_loaded = []
        for image in self.driver.find_elements_by_tag_name('img'):
            loaded = self.driver.execute_script(script, image)
            if not loaded and image.get_attribute('src'):
                images_not_loaded.append('%s: %s' % (self.driver.title, image.get_attribute('src')))
        return images_not_loaded

    def is_element_present(self, locator):
        """
        True if the element at the specified locator is present in the DOM.
        Note: It returns false immediately if the element is not found.
        """
        self.driver.implicitly_wait(0)
        try:
            self.driver.find_element(*locator)
            return True
        except NoSuchElementException:
            return False
        finally:
            # set the implicit wait back
            self.driver.implicitly_wait(self.default_implicit_wait)

    def is_element_visible(self, locator):
        """
        True if the element at the specified locator is visible in the browser.
        Note: It uses an implicit wait if element is not immediately found.
        """
        try:
            return self.driver.find_element(*locator).is_displayed()
        except (NoSuchElementException, ElementNotVisibleException):
            return False

    def is_element_not_visible(self, locator):
        """
        True if the element at the specified locator is not visible.
        Note: It returns true immediately if the element is not found.
        """
        self.driver.implicitly_wait(0)
        try:
            return not self.driver.find_element(*locator).is_displayed()
        except (NoSuchElementException, ElementNotVisibleException):
            return True
        finally:
            # set the implicit wait back
            self.driver.implicitly_wait(self.default_implicit_wait)

    def wait_for_element_present(self, locator, timeout=None):
        """ Wait for the element at the specified locator
        to be present in the DOM. """
        timeout = timeout or self.timeout
        count = 0
        while not self.is_element_present(locator):
            time.sleep(1)
            count += 1
            if count == timeout:
                raise Exception('{0} has not loaded'.format(locator))

    def wait_for_element_visible(self, locator, timeout=None):
        """
        Wait for the element at the specified locator to be visible.
        """
        timeout = timeout or self.timeout
        count = 0
        while not self.is_element_visible(locator):
            time.sleep(1)
            count += 1
            if count == timeout:
                raise Exception("{0} is not visible".format(locator))

    def wait_for_element_not_visible(self, locator, timeout=None):
        """
        Wait for the element at the specified locator not to be visible anymore.
        """
        timeout = timeout or self.timeout
        count = 0
        while self.is_element_visible(locator):
            time.sleep(1)
            count += 1
            if count == timeout:
                raise Exception("{0} is still visible".format(locator))

    def wait_for_element_not_present(self, locator, timeout=None):
        """ Wait for the element at the specified locator
         not to be present in the DOM. """
        timeout = timeout or self.timeout
        self.driver.implicitly_wait(0)
        try:
            WebDriverWait(self.driver, timeout).until(
                lambda s: len(self.find_elements(*locator)) < 1)
            return True
        except TimeoutException:
            Assert.fail(TimeoutException)
        finally:
            self.driver.implicitly_wait(self.default_implicit_wait)

    def wait_for_text_to_match(self, text, locator, max_count=20, delay=0.25):
        """ Waits for element text to match specified text, until certain deadline """
        element = self.driver.find_element(*locator)
        counter = 0
        while element.text != text:
            if counter < max_count:
                time.sleep(delay)
                counter += 1
                element = self.driver.find_element(*locator)
            else:
                Assert.fail('"' + text + '" text did not match "' + element.text
                            + '" after ' + str(counter * delay) + ' seconds')
                break

    def wait_for_attribute_value(self, attribute, attribute_text, locator, max_count=20, delay=0.25):
        """ Waits for element attribute value to match specified text, until certain deadline """
        element = self.driver.find_element(*locator)
        counter = 0
        while element.get_attribute(attribute) != attribute_text:
            if counter < max_count:
                time.sleep(delay)
                counter += 1
            else:
                Assert.fail('"' + attribute_text + '" text did not match "' + element.get_attribute(attribute)
                            + '" after ' + str(counter * delay) + ' seconds')
                break

    def wait_for_element_ready(self, locator, timeout=None):
        """ Waits until certain element is present and clickable """
        timeout = timeout or self.timeout
        WebDriverWait(self.driver, timeout).until(EC.presence_of_element_located(locator),
                                                  'Element specified by {0} was not present!'.format(locator))
        WebDriverWait(self.driver, timeout).until(EC.element_to_be_clickable(locator),
                                                  'Element specified by {0} did not become clickable!'.format(locator))

    def find_element(self, locator):
        """ Return the element at the specified locator."""
        return self.driver.find_element(*locator)

    def find_elements(self, locator):
        """ Return a list of elements at the specified locator."""
        return self.driver.find_elements(*locator)

    def find_elements_with_text(self, text, locator):
        """ Find elements that have specified text """
        elements = self.driver.find_elements(*locator)
        selected = [item for item in elements if item.text == text]
        return selected[0] if len(selected) == 1 else selected


    def link_destination(self, locator):
        """ Return the href attribute of the element at the specified locator."""
        link = self.driver.find_element(*locator)
        return link.get_attribute('href')

    def image_source(self, locator):
        """ Return the src attribute of the element at the specified locator."""
        link = self.driver.find_element(*locator)
        return link.get_attribute('src')

    def select_dropdown_value(self, select, value):
        """ Set 'select' dropdown value """
        select = Select(select)
        option = [option for option in select.options if option.text == value][0]
        option.click()

    def upload_file(self, file_path, input_field_locator, delay=5):
        """ uploads file through the file input field
            @file_path: path to file (including the file name) relative to test project root
            @input_field_locator: locator of input element with type="file"
            @delay: seconds to wait for file to upload
        """
        file_path = os.path.join(self.shishito_support.project_root, file_path)
        self.driver.find_element(*input_field_locator).send_keys(file_path)
        time.sleep(delay)

    def execute_js_script(self, script, arguments=None):
        """execute any js command with arguments or without it"""
        script_value = self.driver.execute_script(script, arguments)
        return script_value

    def open_new_tab(self, url):
        """Open new tab using keyboard, for now work only in Firefox and IE, in Chrome use js script to open tab """
        ActionChains(self.driver).send_keys(Keys.CONTROL, "t").perform()
        windows = self.driver.window_handles
        self.driver.switch_to_window(windows[-1])
        self.driver.get(url)

    def switch_new_tab(self):
        """switch to new tab/window"""
        windows = self.driver.window_handles
        self.driver.switch_to_window(windows[-1])

    def switch_first_tab(self):
        """Close current tab, switch to first tab/window"""
        windows = self.driver.window_handles
        self.driver.close()
        self.driver.switch_to_window(windows[0])


class ClickDelay:
    """
        Class to be used as a decorator that would add sleep after click() calls
        see click_delay() function for details
    """
    def __init__(self, obj):
        self.obj = obj

    def click_delay(self, delay = 2):
        self.obj.click()
        time.sleep(delay)
    
    def __getattr__(self, name):
        if (name == 'click'):
            return self.click_delay
        return getattr(self.obj, name)
    

def click_delay(function):
    """
        Function to be used as a decorator that would add sleep after click() calls.
        Usage scenario:
            this function will decorate a PageDefiniton property/method that returns a webpage Element object.
            The Element object is encapsulated by ClickDelay class (see above) which redefines click() method by adding sleep() to it.
            e.g:
                class LoginPage:

                    @property
                    @click_delay
                    def submit_button(self):
                        return self.driver.find_element_by_css_selector('form button')

                class TestLogin:
                    def test_login_err(self):
                        loginPage.submit_button.click()     # -- will click and sleep(2)
    """
    name = function.__name__
    def wrapper(*args, **kwargs):
        obj = function(*args, **kwargs)
        if (obj == None):
            raise Exception("Error: " + name + "() returned None")
        return ClickDelay(obj)
    return wrapper
 
 




from __future__ import absolute_import

from appium import webdriver
import time

from shishito.runtime.environment.shishito import ShishitoEnvironment


class ControlEnvironment(ShishitoEnvironment):
    """ Appium control environment. """

    def call_browser(self, config_section):
        """ Start webdriver for given config section. Prepare capabilities for the webdriver. If saucelabs setting has value,
        webdriver will be connected to saucelabs. Otherwise appium_url setting will be used.

        :param str config_section: section in platform/environment.properties config
        :return: created webdriver
        """

        # get browser capabilities
        capabilities = self.get_capabilities(config_section)
        browserstack = self.shishito_support.get_opt('browserstack')

        if browserstack:
            try:
                bs_user, bs_password = self.shishito_support.get_opt('browserstack').split(':', 1)
            except (AttributeError, ValueError):
                raise ValueError('Browserstack credentials were not specified! Unable to start browser.')

            if bs_user:
                #remote_url = 'http://%s@ondemand.saucelabs.com:80/wd/hub' % saucelabs
                remote_url = 'http://{0}:{1}@hub-cloud.browserstack.com/wd/hub'.format(bs_user, bs_password)
            else:
                remote_url = self.shishito_support.get_opt('appium_url')

        # get driver
        return self.start_driver(capabilities, remote_url)

    def get_capabilities(self, config_section):
        """ Return dictionary of capabilities for specific config combination.

        :param str config_section: section in platform/environment.properties config
        :return: dict with capabilities
        """

        get_opt = self.shishito_support.get_opt

        capabilities= {
            'os': get_opt(config_section, 'os'),
            'os_version': get_opt(config_section, 'os_version'),
            'device': get_opt(config_section, 'device'),
            'app': get_opt('app') or get_opt(config_section, 'app'),
            'name': self.get_test_name() + time.strftime('_%Y-%m-%d'),
            'browserstack.debug': get_opt('browserstack_debug').lower() or False,
            'browserstack.appium_version':  get_opt(config_section, 'browserstack.appium_version') or '1.7.0',
            'deviceOrientation': get_opt(config_section, 'deviceOrientation') or 'portrait'
        }
        if(get_opt(config_section, 'os')=='android'):
            capabilities['autoGrantPermissions']= get_opt(config_section, 'autoGrantPermissions') or False
        if(get_opt(config_section, 'os')=='ios'):
            capabilities['autoAcceptAlerts'] = get_opt(config_section, 'autoAcceptAlerts').lower() or False
        return capabilities


    def get_pytest_arguments(self, config_section):
        """ Get environment specific arguments for pytest.

        :param config_section: section in platform/environment.properties config
        :return: dict with arguments for pytest or None
        """

        pytest_args = {
            '--os_version': '--os_version=%s' % self.shishito_support.get_opt(config_section, 'os_version'),
            '--device': '--device=%s' % self.shishito_support.get_opt(config_section, 'device'),
            '--app': '--app=%s' % (self.shishito_support.get_opt('app') or self.shishito_support.get_opt(config_section, 'app'))
        }
        browserstack = self.shishito_support.get_opt('browserstack')
        if browserstack:
            pytest_args['--browserstack'] = '--browserstack=%s' % browserstack

        return pytest_args

    def start_driver(self, capabilities, remote_driver_url):
        """ Prepare selenium webdriver.

        :param capabilities: capabilities used for webdriver initialization
        :param remote_driver_url: url to which the driver will be connected
        """

        driver = webdriver.Remote(
            command_executor=remote_driver_url,
            desired_capabilities=capabilities,
        )

        return driver



import imaplib
import email
import time
import re

from shishito.runtime.shishito_support import ShishitoSupport


class EmailIMAP(object):
    def __init__(self):
        self.shishito_support = ShishitoSupport()
        self.email_address = self.shishito_support.get_opt('email_address')
        self.email_imap = self.shishito_support.get_opt('email_imap')
        self.email_password = self.shishito_support.get_opt('email_password')
        self.email_mailbox = self.shishito_support.get_opt('email_mailbox')

        self.mail = imaplib.IMAP4_SSL(self.email_imap)
        self.mail.login(self.email_address, self.email_password)

    def retrieve_latest_email(self, timeout=36):
        """ tries to retrieve latest email. If there are no emails,
        it waits for 5 seconds and then tries again, until total (default 36*5 = 180s) timeout is reached """
        count = 0
        while count < timeout:
            id_list = self.get_all_email_ids()
            if id_list:
                return self.get_message(id_list[-1])
            time.sleep(5)
            count += 1

    def cleanup_emails(self):
        """ cleans up email folder """
        for num in self.get_all_email_ids():
            self.mail.store(num, '+FLAGS', '\\Deleted')
        self.mail.expunge()

    def is_pattern_in_message(self, pattern, message):
        """ return Boolean if pattern is to be found in message """
        return bool(re.match(pattern, message))

    # SUPPORT METHODS

    def get_message(self, message_id):
        """ gets email message """
        data = self.mail.fetch(message_id, "(RFC822)")[1]  # fetch the email body (RFC822) for the given ID
        raw_email = data[0][1]  # here's the body, which is raw text of the whole email
        return email.message_from_string(raw_email)

    def get_all_email_ids(self):
        """ connects to gmail account and return list of all email ids """
        self.mail.select(self.email_mailbox)
        data = self.mail.search(None, "ALL")[1]
        return data[0].split() #ids split



"""
@author: Vojtech Burian
@summary: Selenium Webdriver Python test runner
"""

import argparse
import sys
import time

from shishito.reporting.reporter import Reporter
from shishito.runtime.shishito_support import ShishitoSupport
from shishito.services.testrail_api import TestRail
from shishito.services.qastats_api import QAStats

class ShishitoRunner(object):
    """ Base shishito test runner.

    - runs python selenium tests on customizable configurations (using PyTest)
    - archive the test results in .zip file """

    def __init__(self, project_root):
        # set project root
        self.project_root = project_root

        # test timestamp - for storing results
        self.test_timestamp = time.strftime("%Y-%m-%d_%H-%M-%S")
        self.epoch = int(time.time())

        # parse cmd  args
        self.cmd_args = self.handle_cmd_args()

        # Get SUT build for use in reporting
        self.test_build = self.cmd_args['build']

        self.reporter = Reporter(project_root, self.test_timestamp)
        self.shishito_support = ShishitoSupport(
            cmd_args=self.cmd_args,
            project_root=self.project_root
        )

    def handle_cmd_args(self):
        """ Retrieve command line arguments passed to the script.

        :return: dict with parsed command line arguments
        """

        parser = argparse.ArgumentParser(description='Selenium Python test runner execution arguments.')

        parser.add_argument('--platform',
                            help='Platform on which run tests.',
                            dest='test_platform')
        parser.add_argument('--environment',
                            help='Environment for which run tests.',
                            dest='test_environment')
        parser.add_argument('--test_directory',
                            help='Directory where to lookup for tests')
        parser.add_argument('--smoke',
                            help='Run only smoke tests',
                            action='store_true')
        parser.add_argument('--browserstack',
                            help='BrowserStack credentials; format: "username:token"')
        parser.add_argument('--saucelabs',
                            help='Saucelabs credentials; format: "username:token"')
        parser.add_argument('--test_rail',
                            help='TestRail Test Management tool credentials; format: "username:password"')
        parser.add_argument('--qastats',
                            help='QAStats Test Management tool credentials; format: "token"')
        parser.add_argument('--node_webkit_chromedriver_path',
                            help='Path to chromedriver located in same directory as node-webkit application')
        parser.add_argument('--app',
                            help='Path to appium application')
        parser.add_argument('--test',
                            help='Run specified test (PyTest string expression)')
        parser.add_argument('--build',
                            help='Specify build number for reporting purposes')
        parser.add_argument('--maxfail',
                            help='stop after x failures')
        args = parser.parse_args()

        # return args dict --> for use in other classes
        return vars(args)

    def run_tests(self):
        """ Execute tests for given platform and environment. Platform and Environment can be passed as command lines
        argument or settings in config file.
        """

        if __name__ == "__main__":
            sys.exit('The runner cannot be executed directly.'
                     ' You need to import it within project specific runner. Session terminated.')

        # cleanup previous results
        self.reporter.cleanup_results()

        # import execution class
        executor_class = self.shishito_support.get_module('platform_execution')
        # executor_class = getattr(import_module(platform_path), 'ControlExecution')
        executor = executor_class(self.shishito_support, self.test_timestamp)

        # run test
        exit_code = executor.run_tests()

        # archive results + generate combined report
        self.reporter.archive_results()
        self.reporter.generate_combined_report()

        # upload results to QAStats test management app
        qastats_credentials = self.shishito_support.get_opt('qastats')
        if qastats_credentials:
            try:
                qas_user, qas_password = qastats_credentials.split(':', 1)
            except (AttributeError, ValueError):
                raise ValueError('QAStats credentials were not specified! Unable to connect to QAStats.')

            qastats = QAStats(qas_user, qas_password, self.test_timestamp, self.epoch, self.test_build)
            qastats.post_results()

        # upload results to TestRail test management app
        test_rail_credentials = self.shishito_support.get_opt('test_rail')
        if test_rail_credentials:
            try:
                tr_user, tr_password = test_rail_credentials.split(':', 1)
            except (AttributeError, ValueError):
                raise ValueError('TestRail credentials were not specified! Unable to connect to TestRail.')

            test_rail = TestRail(tr_user, tr_password, self.test_timestamp, self.test_build)
            test_rail.post_results()

        return exit_code



import glob
import os
import shutil
import time
import xml.etree.ElementTree as ET

from jinja2 import Environment, FileSystemLoader

from shishito.runtime.shishito_support import ShishitoSupport


class Reporter(object):
    def __init__(self, project_root=None, test_timestamp=None):
        self.project_root = project_root or ShishitoSupport().project_root
        self.current_folder = os.path.dirname(os.path.abspath(__file__))
        self.timestamp = test_timestamp or time.strftime("%Y-%m-%d_%H-%M-%S")

        self.result_folder = os.path.join(self.project_root, 'results', self.timestamp)

    def cleanup_results(self):
        """ Cleans up test result folder """
        if os.path.exists(os.path.join(self.project_root, 'results')):
            shutil.rmtree(os.path.join(self.project_root, 'results'))
        os.makedirs(self.result_folder)
        os.symlink(self.timestamp, os.path.join(self.project_root, 'results', 'current'))

    def archive_results(self):
        """ Archives test results in zip package """
        archive_folder = os.path.join(self.project_root, 'results_archive')
        if not (os.path.exists(archive_folder)):
            os.makedirs(archive_folder)
        shutil.make_archive(os.path.join(archive_folder, self.timestamp), "zip",
                            os.path.join(self.project_root, 'results'))

    def generate_combined_report(self):
        data = os.listdir(os.path.join(self.project_root, 'results', self.timestamp))
        result_reports = [item[:-5] for item in data if item.endswith('.html')]

        if not result_reports:
            return

        env = Environment(
            loader=FileSystemLoader(os.path.join(self.current_folder, 'resources'))
        )

        template = env.get_template('CombinedReportTemplate.html')
        template_vars = {'data': result_reports}
        output = template.render(template_vars)
        formatted_output = output.encode('utf8').strip()
        final_report = open(os.path.join(self.project_root, 'results', self.timestamp, 'CombinedReport.html'), 'w')
        final_report.write(formatted_output.decode('utf8'))
        final_report.close()
        shutil.copy(
            os.path.join(self.current_folder, 'resources', 'combined_report.js'),
            os.path.join(self.project_root, 'results', self.timestamp)
        )

    def get_xunit_test_cases(self, timestamp):
        """ Parses test names and results from xUnit result file

        :return: Dictionary of test-case (name, result) for each report file """
        result_folder = os.path.join(self.project_root, 'results', timestamp)
        files = glob.glob(os.path.join(result_folder, '*.xml'))
        test_cases = []
        for result_file in files:
            case = {'name': os.path.basename(result_file), 'cases': []}
            tree = ET.parse(result_file)
            root = tree.getroot()
            for child in root:
                if child.tag == 'testcase':
                    entry = {'name': child.get('name')}
                    result = 'success'
                    failure_message = ''
                    for subChild in child:
                        if failure_message != '': failure_message += '\n'
                        failure_message += subChild.text
                        if subChild.tag == 'failure' and result == 'success':
                            result = 'failure'
                        elif subChild.tag == 'error' and result == 'success':
                            result = 'error'
                        elif subChild.tag == 'skipped' and result == 'success':
                            result = 'skipped'
                    entry['result'] = result
                    if result == 'success':
                        entry['failure_message'] = None
                    else:
                        entry['failure_message'] = failure_message
                    case['cases'].append(entry)
            test_cases.append(case)
        return test_cases



"""
@summary: Common configuration functions supporting test execution.
 Various startup and termination procedures, helper functions etc.
 Not to be used for directly testing the system under test (must not contain Asserts etc.)
"""

from shishito.runtime.platform.shishito_control_test import ShishitoControlTest



class ControlTest(ShishitoControlTest):
    """ ControlTest for mobile platform """
    def test_init(self, url):
        """ Executed only once after browser starts.
         Suitable for general pre-test logic that do not need to run before every individual test-case.

        :param str url:
        """
        test_platform = self.shishito_support.test_platform
        test_env = self.shishito_support.get_opt('test_environment')
        if(test_env == 'browserstack' and test_platform == 'mobile'):
            self.driver.get(url)
            self.driver.implicitly_wait(int(self.shishito_support.get_opt('default_implicit_wait')))





from shishito.runtime.platform.shishito_execution import ShishitoExecution


class ControlExecution(ShishitoExecution):
    """ ControlExecution for node-webkit platform"""

    def get_test_result_prefix(self, config_section):
        """ Create string prefix for test results.

        :param str config_section: section in platform/environment.properties config
        :return: str with test result prefix
        """

        return 'node_webkit'




from setuptools import find_packages, setup
from setuptools.command.install import install
import os
import sys

VERSION = '3.0.5'

with open('README.md', encoding='utf-8') as readme_file:
    readme = readme_file.read()

with open('HISTORY.md', encoding='utf-8') as history_file:
    history = history_file.read()

class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""
    description = 'verify that the git tag matches our version'

    def run(self):
        tag = os.getenv('CIRCLE_TAG')

        if tag != VERSION:
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, VERSION
            )
            sys.exit(info)


setup(
    name='shishito',
    version=VERSION,
    url='https://github.com/salsita/shishito',
    description='Python module for selenium webdriver test execution',
    long_description=readme + '\n\n' + history,
    long_description_content_type='text/markdown',
    author='Vojtech Burian',
    author_email='vojtech.burian@gmail.com',
    license='MIT',
    platforms='any',
    keywords=['testing', 'selenium', 'webdriver', 'browserstack', 'appium'],
    packages=find_packages(),
    package_data={
        'shishito': ['reporting/resources/*.js', 'reporting/resources/*.css', 'reporting/resources/*.html']
    },
    install_requires=[
        'selenium', 'pytest-xdist', 'pytest-instafail', 'pytest', 'UnittestZero',
        'Jinja2', 'requests', 'Appium-Python-Client', 'Click>=6.0'
    ],
    scripts=['shi'],
    entry_points={
        'console_scripts': [
            'shishito=shishito.cli:main',
        ],
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Topic :: Software Development :: Testing",
    ],
    cmdclass={
        'verify': VerifyVersionCommand,
    }
)



"""
@author: Vojtech Burian
@summary: Selenium Webdriver Python test runner
"""

import os
import sys
import pytest

class CollectionPlugin:
    # collect test data from pytest

    def __init__(self):
        self.collected = []

    def pytest_collection_modifyitems(self, items):
        for item in items:
            self.collected.append((item.cls, item.name))

class ShishitoExecution(object):
    """ Base class for platform ControlExecution objects. """

    def __init__(self, shishito_support, test_timestamp):
        self.shishito_support = shishito_support

        environment_class = self.shishito_support.get_module('test_environment')
        self.environment = environment_class(shishito_support)

        self.result_folder = os.path.join(self.shishito_support.project_root, 'results', test_timestamp)

    def get_test_result_prefix(self, config_section):
        """ Create string prefix for test results.

        :param str config_section: section in platform/environment.properties config
        :return: str with test result prefix
        """

        return ''

    def run_tests(self):
        """ Trigger PyTest runner.
        Run PyTest for each browser/device combination, taken from .properties file for proper
        platform and environment.
        """

        test_status = 0

        for config_section in self.shishito_support.env_config.sections():

            print('Running combination: ' + config_section)
            test_status = self.trigger_pytest(config_section)

        return test_status

    def collect_tests(self):
        # collect tests discoverable by pytest

        test_directory = self.shishito_support.get_opt('test_directory')

        collect_plugin = CollectionPlugin()

        pytest_arguments = [
            os.path.join(self.shishito_support.project_root, test_directory),
        ]

        pytest_arguments.extend(['--collect-only'])
        pytest_arguments.extend(['-p', 'no:terminal'])
        pytest.main(pytest_arguments, plugins=[collect_plugin])

        return collect_plugin.collected

    def trigger_pytest(self, config_section):
        """ Run PyTest runner on specific browser/device configuration. Function creates arguments for pytest.
        Function executes pytest.main() with created arguments.

        :param str config_section: section in platform/environment.properties config
        :return: result of pytest.main() function
        """

        test_result_prefix = self.get_test_result_prefix(config_section)

        junit_xml_path = os.path.join(self.result_folder, config_section + '.xml')
        html_path = os.path.join(self.result_folder, config_section + '.html')

        # prepare pytest arguments into execution list
        pytest_arguments_dict = {
            '--test_platform=': '--test_platform=' + self.shishito_support.test_platform,
            '--test_environment=': '--test_environment=' + self.shishito_support.test_environment,
            '--environment_configuration=': '--environment_configuration=' + config_section,
            '--junitxml=': '--junitxml=' + junit_xml_path,
            '--junit-prefix=': '--junit-prefix=' + test_result_prefix,
            '--html=': '--html=' + html_path,
            '--html-prefix=': '--html-prefix=' + test_result_prefix,
            '--instafail': '--instafail',
        }
        # extend pytest_arguments with environment specific args
        extra_pytest_arguments = self.environment.get_pytest_arguments(config_section)
        if extra_pytest_arguments:
            pytest_arguments_dict.update(extra_pytest_arguments)

        test_directory = self.shishito_support.get_opt('test_directory')
        if not test_directory:
            raise ValueError('Not test directory was specified.')

        pytest_arguments = [
            os.path.join(self.shishito_support.project_root, test_directory),
        ]

        pytest_arguments.extend(pytest_arguments_dict.values())
        if sys.version_info.major > 2:
            pytest_arguments.extend(['-p', 'pytest_imports'])   # import parser addoption to support extra options

        # set pytest parallel execution argument
        parallel_tests = int(self.shishito_support.get_opt('parallel_tests'))
        if parallel_tests > 1:
            pytest_arguments.extend(['-n', str(parallel_tests)])

        # set pytest maxfail argument
        maxfail = self.shishito_support.get_opt('maxfail')

        if maxfail:
            pytest_arguments.extend(['--maxfail={}'.format(maxfail)])

        # set pytest smoke test argument
        smoke = self.shishito_support.get_opt('smoke')
        if smoke:
            pytest_arguments.extend(['-m', 'smoke'])

        # run only specific tests
        test_stringexpr = self.shishito_support.get_opt('test')
        if test_stringexpr:
            pytest_arguments.extend(['-k', test_stringexpr])

        # verbose diffs
        pytest_arguments.extend(['-vv'])

        # Uncomment following to enable stdout and stderr (otherwise it is added to the final report)
        # pytest_arguments.extend(['-s'])

        return pytest.main(pytest_arguments)



"""
@author: Vojtech Burian
@summary: Fixtures for handling custom pytest parameters
"""
import pytest

from shishito.reporting.junithtml import LogHTML


# CURRENT TEST INFO OBJECT #

class TestInfo():
    """ Gathers test execution information about currently running test """

    def __init__(self):
        self.test_status = None
        self.test_name = None

    def set_test_info(self, new_status=None, new_name=None):
        self.test_status = new_status
        self.test_name = new_name


test_info = TestInfo()


def get_test_info():
    return test_info


# FIXTURES #

def pytest_addoption(parser):
    parser.addoption("--url", action="store", default="https://salsitasoft.com", help="specify URL for test site")

    # MODULES
    parser.addoption('--test_platform', action="store",
                     help="Test Platform")
    parser.addoption('--test_environment', action="store",
                     help="Test Environment")
    parser.addoption('--environment_configuration', action="store",
                     help="Environment Configuration")

    # BROWSERSTACK
    parser.addoption("--browser", action="store",
                     help="BrowserStack browser: Chrome, Firefox, IE, PhantomJS, Opera, Safari")
    parser.addoption("--browser_version", action="store",
                     help="BrowserStack browser version. Value depends on selected browser.")
    parser.addoption("--os", action="store",
                     help="BrowserStack operating system: Windows, OS X")
    parser.addoption("--os_version", action="store",
                     help="BrowserStack operating system version. Depends on selected OS.")
    parser.addoption("--resolution", action="store",
                     help="Screen resolution: 1024x768, 1280x960, 1280x1024, 1600x1200, 1920x1080")

    parser.addoption("--xbuildname", action="store", default="Unnamed",
                     help="Test build name")
    parser.addoption("--xbrowserName", action="store", default="iPad",
                     help="BrowserStack mobile browser: iPad, iPhone, android")
    parser.addoption("--xplatform", action="store", default="MAC",
                     help="BrowserStack mobile platform: MAC, ANDROID")
    parser.addoption("--xdevice", action="store", default="iPad Air",
                     help="BrowserStack mobile device: iPad Air, Samsung Galaxy S5 and others")
    parser.addoption("--deviceOrientation", action="store", default="portrait",
                     help="BrowserStack mobile device screen orientation: portrait or landscape")
    parser.addoption("--jira_support", action="store", default=None,
                     help="Jira username and password")
    parser.addoption('--jira_cycle_id', action="store", default=None,
                     help="Set JIRA cycle id")
    parser.addoption("--browserstack", action="store", default=None,
                     help="Browserstack username and password")
    parser.addoption('--test_mobile', action="store", default=None,
                     help="Execute tests on mobile devices")



    # APPIUM

    parser.addoption('--platformName', action="store", default=None,
                     help="Appium platform: ios, android")
    parser.addoption('--platformVersion', action="store", default=None,
                     help="Appium platform version. For iOS e.g. '7.1', for Android e.g. '4.4'")
    parser.addoption('--deviceName', action="store", default=None,
                     help="Device name (simulator or real device)")
    parser.addoption('--appiumVersion', action="store", default=None,
                     help="Appium verison, e.g. '1.3.5'")
    parser.addoption('--appPackage', action="store", default=None,
                     help="Android app package, e.g. 'com.example.android.notepad'")
    parser.addoption('--appActivity', action="store", default=None,
                     help="Android default activity, e.g. '.NotesList'")
    parser.addoption('--app', action="store", default=None,
                     help="File path or link to iOS app.zip file / Android apk file")
    parser.addoption("--autoAcceptAlerts", action="store", default=True,
                     help="Auto accepts alerts on iOS devices")
    parser.addoption("--saucelabs", action="store", default=None,
                     help="Saucelabs username and password")

    # BROWSERSTACK MOBILE

    parser.addoption('--platform', action="store", default=None,
                     help="Appium platform: ios, android")
    parser.addoption('--device', action="store", default=None,
                     help="Device name (simulator or real device)")
    parser.addoption('--browserstack.appium_version', action="store", default=None,
                     help="Appium default version")

    # REPORTS
    group = parser.getgroup("terminal reporting")
    group.addoption('--html', '--junit-html', action="store",
                    dest="htmlpath", metavar="path", default=None,
                    help="create html style report file at given path.")
    group.addoption('--htmlprefix', '--html-prefix', action="store",
                    dest="prefix", metavar="str", default=None,
                    help="prepend prefix to classnames in html output")

    # TESTRAIL
    parser.addoption("--test_rail", action="store", default=None,
                     help="TestRail Test Management tool credentials")

    # QAStats
    parser.addoption("--qastats", action="store", default=None,
                     help="QAStats Test Management tool credentials")


def pytest_configure(config):
    htmlpath = config.option.htmlpath
    prefix = config.option.prefix
    # prevent opening xmllog on slave nodes (xdist)
    if htmlpath and not hasattr(config, 'slaveinput'):
        config._html = LogHTML(htmlpath, prefix)
        config.pluginmanager.register(config._html)


def pytest_unconfigure(config):
    html = getattr(config, '_html', None)
    if html:
        del config._html
        config.pluginmanager.unregister(html)


def pytest_collection_finish(session):
    test_names = {}
    for i in session.items:
        name, module = i.name, i.cls.__module__
        if name in test_names:
            raise Exception("You should rename duplicate test method: '%s'. It was found in modules: '%s' and '%s'"%(name, module, test_names[name]))
        test_names[name] = module


@pytest.hookimpl(hookwrapper=True)
def pytest_runtest_makereport(item, call):
    # execute all other hooks to obtain the report object
    outcome = yield
    rep = outcome.get_result()
    # set an report attribute for each phase of a call, which can
    # be "setup", "call", "teardown"
    setattr(item, "rep_" + rep.when, rep)
    return rep


@pytest.fixture()
def test_status(request):
    test_name = request.node.nodeid.split('::')[-1]

    def fin():
        # request.node is an "item" because we use the default
        # "function" scope
        if request.node.rep_setup.failed:
            print("setting up a test failed!", request.node.nodeid)
            test_info.set_test_info('failed_setup', test_name)
        elif request.node.rep_setup.passed:
            if request.node.rep_call.failed:
                test_info.set_test_info('failed_execution', test_name)
                print("executing test failed", request.node.nodeid)
            elif "xfail" in request.node.keywords:
                test_info.set_test_info('expected_fail', test_name)
                print("executing test failed as eXpected", request.node.nodeid)

    request.addfinalizer(fin)
    test_info.set_test_info('passed', test_name)


@pytest.fixture(scope='class')
def browser(request):
    return request.config.getoption("--browser")


@pytest.fixture
def browser_version(request):
    return request.config.getoption("--browser_version")


@pytest.fixture
def os(request):
    return request.config.getoption("--os")


@pytest.fixture
def os_version(request):
    return request.config.getoption("--os_version")


@pytest.fixture
def resolution(request):
    return request.config.getoption("--resolution")


@pytest.fixture
def xbuildname(request):
    return request.config.getoption("--xbuildname")


@pytest.fixture
def xdevice(request):
    return request.config.getoption("--xdevice")


@pytest.fixture
def xdeviceOrientation(request):
    return request.config.getoption("--xdeviceOrientation")


@pytest.fixture
def xbrowserName(request):
    return request.config.getoption("--xbrowserName")


def xplatform(request):
    return request.config.getoption("--xplatform")


@pytest.fixture
def jira_support(request):
    return request.config.getoption("--jira_support")


@pytest.fixture
def browserstack(request):
    return request.config.getoption("--browserstack")


@pytest.fixture
def jira_cycle_id(request):
    return request.config.getoption("--jira_cycle_id")


@pytest.fixture
def test_mobile(request):
    return request.config.getoption("--test_mobile")


@pytest.fixture(scope='class')
def url(request):
    return request.config.getoption("--url")
